@@ -68,6 +68,11 @@ except ImportError:
 	# 0.8
 	import gst
 	pygst = None
+
+if gst.gst_version < (0,10,0):
+	print "%s needs GStreamer 0.10, you have version: %s" % (NAME, ".".join([str(s) for s in gst.gst_version]))
+	sys.exit(1)
+
 print "  using Gstreamer version: %s, Python binding version: %s" % (
 		".".join([str(s) for s in gst.gst_version]), 
 		".".join([str(s) for s in gst.pygst_version]) )
@@ -92,6 +97,7 @@ Guillaume Bedot <guillaume.bedot wanadoo.fr> (French)
 Dominik Zabłotny <dominz wp.pl> (Polish) 
 Jonh Wendell <wendell bani.com.br> (Portuguese Brazilian)
 Marc E. <m4rccd yahoo.com> (Spanish)
+Daniel Nylander <po danielnylander se> (Swedish)
 """)
 
 # Names of columns in the file list
@@ -198,19 +204,37 @@ def file_encode_filename(filename):
 	filename = filename.replace(" ", "\ ");
 	return filename
 	
+
+def unquote_filename(filename):
+
+	f= urllib.unquote(filename)
+	try:
+		# files are normaly in utf-8 ?
+		f = unicode(f, "utf-8")
+	except UnicodeDecodeError:
+		# but sometimes they are badly encoded, this is a failback
+		f = unicode(f, "latin1", "replace")
+	return f
+
+
 use_gnomevfs = False
 
-def markup_escape(str):
-    str = "&amp;".join(str.split("&"))
-    str = "&lt;".join(str.split("<"))
-    str = "&gt;".join(str.split(">"))
-    return str
+def markup_escape(message):
+	if not isinstance(message, basestring):
+		print "markup error: '%s'" % message
+		import pdb
+		pdb.set_trace()
+
+	message = "&amp;".join(message.split("&"))
+	message = "&lt;".join(message.split("<"))
+    	message = "&gt;".join(message.split(">"))
+    	return message
 
 def filename_escape(str):
-    str = str.replace("'","\'")
-    str = str.replace("\"","\\\"")
-    str = str.replace("!","\!")
-    return str
+	str = str.replace("'","\'")
+	str = str.replace("\"","\\\"")
+	str = str.replace("!","\!")
+	return str
 
 if gst.element_factory_find("gnomevfssrc"):
 	gstreamer_source = "gnomevfssrc"
@@ -282,7 +306,7 @@ class SoundFile:
 		else:
 			self.base_path, self.filename = os.path.split(self.uri)
 			self.base_path += "/"
-		self.filename_for_display = gnomevfs.unescape_string_for_display(self.filename)
+		self.filename_for_display = unquote_filename(self.filename)
 	
 		self.tags = {
 			"track-number": 0,
@@ -395,7 +419,7 @@ class TargetNameGenerator:
 		result = urllib.quote(pattern % dict)
 		if self.replace_messy_chars:
 			s = ""
-			result = urllib.unquote(result)
+			result = unquote_filename(result)
 			for c in result:
 				if c not in self.nice_chars:
 					s += "_"
@@ -488,18 +512,14 @@ class BackgroundTask:
 	def thread_work(self):
 		working = True
 		while self and working:
-			#gtk.threads_enter()
 			working = self.do_work_()
-			#gtk.threads_leave()
 			sleep(_thread_sleep)
 			while gtk.events_pending():
 				gtk.main_iteration()
 
 
 	def do_work(self):
-		#gtk.threads_enter()
 		working = self.do_work_()
-		#gtk.threads_leave()
 		return working
 
 
@@ -570,8 +590,8 @@ class TaskQueue(BackgroundTask):
 		BackgroundTask.__init__(self)
 		self.tasks = []
 		self.running = False
-		self.tasks_number=0
-		self.tasks_current=0
+		self.tasks_current = 0
+		self.tasks_number = 0
 
 	def is_running(self):
 		return self.running
@@ -615,6 +635,7 @@ class TaskQueue(BackgroundTask):
 		self.running = False
 		log("Queue done in %ds" % (time.time() - self.start_time))
 		self.queue_ended()
+		self.tasks_number = 0
 
 	def stop(self):
 		if self.tasks:
@@ -622,6 +643,7 @@ class TaskQueue(BackgroundTask):
 		BackgroundTask.stop(self)
 		self.running = False
 		self.tasks = []
+		self.tasks_number = 0
 
 	# The following hooks are called after each sub-task has been set up,
 	# after its work method has been called, and after it has finished.
@@ -666,6 +688,7 @@ class Pipeline(BackgroundTask):
 		self.signals = []
 		self.processing = False
 		self.eos = False
+		self.error = None
 		
 	def setup(self):
 		#print "Pipeline.setup()"
@@ -708,7 +731,8 @@ class Pipeline(BackgroundTask):
 		if t == gst.MESSAGE_ERROR:
 			err, debug = message.parse_error()
 			self.eos = True
-			log("error:%s (%s)" % (err, self.sound_file.get_filename_for_display()))
+			self.error = err
+			log("error: %s (%s)" % (err, self.sound_file.get_filename_for_display()))
 		elif t == gst.MESSAGE_EOS:
 			self.eos = True
 		if message.type.value_nicks[1] == "tag":
@@ -765,7 +789,7 @@ class TypeFinder(Pipeline):
 			if t in mime_type:
 				self.sound_file.mime_type = mime_type
 		if not self.sound_file.mime_type:
-			log("Mime type skipped: %s (mail us if this is an error)" % mime_type)
+			log("Mime type skipped: %s" % mime_type)
 	
 	def work(self):
 		return Pipeline.work(self) and not self.sound_file.mime_type
@@ -836,7 +860,6 @@ class Decoder(Pipeline):
 	def get_duration(self):
 		""" return the total duration of the sound file """
 		if not self.pipeline:
-			print "no pipeline"
 			return 0
 		self.query_duration()
 		return self.sound_file.duration
@@ -918,7 +941,7 @@ class Converter(Decoder):
 
 	"""A background task for converting files to another format."""
 
-	def __init__(self, sound_file, output_filename, output_type):
+	def __init__(self, sound_file, output_filename, output_type, delete_original=False):
 		#print "Converter()"
 		Decoder.__init__(self, sound_file)
 
@@ -932,6 +955,7 @@ class Converter(Decoder):
 		self.mp3_quality = None
 
 		self.overwrite = False
+		self.delete_original = delete_original
 
 	#def setup(self):
 	#	self.init()
@@ -990,6 +1014,10 @@ class Converter(Decoder):
 		except:
 			log(_("Cannot set permission on '%s'") % gnomevfs.format_uri_for_display(self.output_filename))
 
+		if self.delete_original and self.processing and not self.error:
+			log("deleting: '%s'" % self.sound_file.get_uri())
+			gnomevfs.unlink(self.sound_file.get_uri())
+
 	def get_position(self):
 		return self.position
 	
@@ -1037,7 +1065,7 @@ class Converter(Decoder):
 			
 			cmd += "%s=%s " % (properties[self.mp3_mode][1], self.mp3_quality)
 	
-		return cmd
+		return cmd + " ! id3v2mux "
 
 class FileList:
 	"""List of files added by the user."""
@@ -1164,7 +1192,7 @@ class FileList:
 			self.typefinders.run()
 
 	def typefinder_queue_ended(self):
-		print "typefinder_queue_ended"
+		pass
 
 	def format_cell(self, sound_file):
 		
@@ -1175,7 +1203,7 @@ class FileList:
 							% _("no tags")
 
 		params = {}
-		params["filename"] = markup_escape(urllib.unquote(sound_file.get_filename_for_display()))
+		params["filename"] = markup_escape(unquote_filename(sound_file.get_filename()))
 		for item in ("title", "artist", "album"):
 			params[item] = markup_escape(sound_file.get_tag(item))
 		if sound_file["bitrate"]:
@@ -1183,27 +1211,17 @@ class FileList:
 		else:
 			params["bitrate"] = ""
 
-		try:	
-			if sound_file.have_tags:
-				template = template_tags
+
+		if sound_file.have_tags:
+			template = template_tags
+		else:
+			if sound_file.tags_read:
+				template = template_notags
 			else:
-				if sound_file.tags_read:
-					template = template_notags
-				else:
-					template = template_loading
+				template = template_loading
 
-			s = template % params
-		except UnicodeDecodeError:
-			str = ""
-			for c in markup_escape(urllib.unquote(sound_file.get_uri())):
-				if ord(c) < 127:
-					str += c
-				else:
-					str += '<span foreground="yellow" background="red"><b>!</b></span>'
-
-			error.show(_("Invalid character in filename!"), str)
-			sys.exit(1)
-				
+		s = template % params
+			
 		return s
 
 	def append_file(self, sound_file):
@@ -1274,6 +1292,7 @@ class PreferencesDialog:
 		"mp3-cbr-quality": 192,
 		"mp3-abr-quality": 192,
 		"mp3-vbr-quality": 3,
+		"delete-original": 0,
 	}
 
 	sensitive_names = ["vorbis_quality", "choose_folder", "create_subfolders",
@@ -1291,6 +1310,7 @@ class PreferencesDialog:
 		self.example = glade.get_widget("example_filename")
 		self.aprox_bitrate = glade.get_widget("aprox_bitrate")
 		self.quality_tabs = glade.get_widget("quality_tabs")
+		self.delete_original = glade.get_widget("delete_original")
 
 		self.target_bitrate = None
 		self.convert_setting_from_old_version()
@@ -1306,42 +1326,12 @@ class PreferencesDialog:
 	def convert_setting_from_old_version(self):
 		""" try to convert previous settings"""
 		
-		# TODO: why not just reseting the settings if we cannot load them ?
-			
-		# vorbis quality was stored as an int enum
+		# vorbis quality was once stored as an int enum
 		try:
 			self.get_float("vorbis-quality")
 		except gobject.GError:
-			log("converting old vorbis setting...")
-			old_quality = self.get_int("vorbis-quality")
-			self.gconf.unset(self.path("vorbis-quality"))
-			quality_setting = (0,0.2,0.3,0.6,0.8)
-			self.set_float("vorbis-quality", quality_setting[old_quality])
-			
-		# mp3 quality was stored as an int enum
-		cbr = self.get_int("mp3-cbr-quality")
-		if cbr <= 4:
-			log("converting old mp3 quality setting... (%d)" % cbr)
-
-			abr = self.get_int("mp3-abr-quality")
-			vbr = self.get_int("mp3-vbr-quality")
-
-			cbr_quality = (64, 96, 128, 192, 256)
-			vbr_quality = (9, 7, 5, 3, 1)
-
-			self.set_int("mp3-cbr-quality", cbr_quality[cbr])
-			self.set_int("mp3-abr-quality", cbr_quality[abr])
-			self.set_int("mp3-vbr-quality", vbr_quality[vbr])
-
-		# mp3 mode was stored as an int enum
-		try:
-			self.get_string("mp3-mode")
-		except gobject.GError:
-			log("converting old mp3 mode setting...")
-			old_mode = self.get_int("mp3-mode")
-			self.gconf.unset(self.path("mp3-mode"))
-			modes = ("cbr","abr","vbr")
-			self.set_string("mp3-mode", modes[old_mode])
+			log("deleting old settings...")
+			[self.gconf.unset(self.path(k)) for k in self.defaults.keys()]
 
 		self.gconf.clear_cache()
 
@@ -1374,6 +1364,9 @@ class PreferencesDialog:
 			w = glade.get_widget("replace_messy_chars")
 			w.set_active(True)
 
+		if self.get_int("delete-original"):
+			self.delete_original.set_active(True)
+
 		mime_type = self.get_string("output-mime-type")
 
 		# desactivate mp3 output if encoder plugin is not present
@@ -1507,7 +1500,7 @@ class PreferencesDialog:
 		generator.set_basename_pattern(self.get_basename_pattern())
 		if for_display:
 			generator.set_replace_messy_chars(False)
-			return urllib.unquote(generator.get_target_name(sound_file))
+			return unquote_filename(generator.get_target_name(sound_file))
 		else:
 			generator.set_replace_messy_chars(
 				self.get_int("replace-messy-chars"))
@@ -1569,6 +1562,12 @@ class PreferencesDialog:
 		self.dialog.run()
 		self.dialog.hide()
 
+	def on_delete_original_toggled(self, button):
+		if button.get_active():
+			self.set_int("delete-original", 1)
+		else:
+			self.set_int("delete-original", 0)
+			
 	def on_same_folder_as_input_toggled(self, button):
 		if button.get_active():
 			self.set_int("same-folder-as-input", 1)
@@ -1742,7 +1741,7 @@ class ConverterQueue(TaskQueue):
 	
 		output_filename = self.window.prefs.generate_filename(sound_file)
 		path = urlparse.urlparse(output_filename) [2]
-		path = urllib.unquote(path)
+		path = unquote_filename(path)
 	
 		exists = True
 		try:
@@ -1796,7 +1795,8 @@ class ConverterQueue(TaskQueue):
 				raise ConverterQueueCanceled()
 			
 		c = Converter(sound_file, output_filename, 
-					  self.window.prefs.get_string("output-mime-type"))
+					  self.window.prefs.get_string("output-mime-type"),
+					  self.window.prefs.get_int("delete-original"))
 		c.set_vorbis_quality(self.window.prefs.get_float("vorbis-quality"))
 		
 		quality = {
@@ -1809,7 +1809,8 @@ class ConverterQueue(TaskQueue):
 		c.set_mp3_quality(self.window.prefs.get_int(quality[mode]))
 		c.init()
 		TaskQueue.add(self, c)
-		self.total_duration += c.get_duration()
+		c.got_duration = False
+		#self.total_duration += c.get_duration()
 
 	def work_hook(self, task):
 		gobject.idle_add(self.set_progress, (task))
@@ -1823,13 +1824,20 @@ class ConverterQueue(TaskQueue):
 		if t:
 			f = t.sound_file.get_filename_for_display()
 
+		for t in self.tasks:
+			if not t.got_duration:
+				duration = t.sound_file.duration
+				if duration: 
+					self.total_duration += duration
+					t.got_duration = True
+
+		print "%d + %d / %d" % ( self.duration_processed , task.get_position(), self.total_duration )
 		self.window.set_progress(self.duration_processed + task.get_position(),
 							 self.total_duration, f)
 		return False
 
 	def finish_hook(self, task):
-		#print "finished: %d+=%d" % (self.duration_processed, task.get_duration())
-		self.duration_processed += task.get_duration()
+		self.duration_processed += task.sound_file.duration
 
 	def finish(self):
 		TaskQueue.finish(self)
@@ -1860,6 +1868,7 @@ class ConverterQueue(TaskQueue):
 		TaskQueue.stop(self)
 		self.window.set_progress(0, 0)
 		self.window.set_sensitive()
+		self.reset_counters()
 
 class CustomFileChooser:
 	"""
@@ -2179,7 +2188,7 @@ class SoundConverterWindow:
 		
 		self.progressfile.set_markup("<i><small>%s</small></i>" % current_file)
 		fraction = float(done_so_far) / total
-		
+	
 		self.progressbar.set_fraction( min(fraction, 1.0) )
 		t = time.time() - self.converter.run_start_time - self.converter.paused_time
 		
@@ -2218,6 +2227,7 @@ def cli_tags_main(input_files):
 	global error
 	error = ErrorPrinter()
 	for input_file in input_files:
+		input_file = SoundFile(input_file)
 		if not get_option("quiet"):
 			print input_file.get_uri()
 		t = TagReader(input_file)
